# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Helpers to manipulate a tensor graph in python.

"""

import sys as _sys

from tensorflow.python.framework.importer import import_graph_def